
; COLD BOOT PROGRAM FOR DEC VT-180
;
; LOADS CCP, BDOS, AND BIOS INTO MEMORY FROM SYSTEM TRACKS
; READ INTO LOCATION 4000H BY ROM BOOT
;
	TITLE	'VT-180 COLD BOOT (TRACK 0, SECTOR 1)'
;
;
INI	MACRO			;;LOAD MEMORY (H,L) WITH INPUT PORT
	DB	0EDH,0A2H	;;	(SPECIFIED BY REG C)  INC H,L  DEC B
	ENDM
;
; SYSTEM ADDRESSES FOR LOAD AND START
;
KMSIZE	EQU	62	; MEMORY SIZE IN KBYTES
;
LCCP	EQU	(KMSIZE-20)*1024+3400H  ;  LOAD ADDRESS
LBIOS	EQU	LCCP+1600H	; BIOS COLD START ENTRY
CDISK	EQU	0004		; Current disk storage
;
; DISK CONSTANTS
;
KSECTRK EQU	9	; SECTORS PER TRACK
KSECSIZ EQU	512	; BYTES PER SECTOR
KBIOLEN EQU	3*1024	; LENGTH OF BIOS TO BE LOADED
KNSECTS EQU	(LBIOS+KBIOLEN-LCCP)/KSECSIZ+1 ; NR OF SECTORS TO LOAD
;
; VT-180 PORT ADDRESSES
;
PCOMD	EQU	78H	; FDC COMMAND/STATUS REG
PTRKRG	EQU	79H	; FDC TRACK REG
PSECRG	EQU	7AH	; FDC SECTOR REG
PDATA	EQU	7BH	; FDC DATA REG
PSTAT	EQU	80H	; GENERAL FD STATUS REG
PCONT	EQU	98H	; ROM CONTROL PORT
;
; DISK CONTROLLER COMMANDS
;
CREST	EQU	0BH	; RESTORE (HEAD LOAD, SLOW STEP)
CSTEPIN EQU	5BH	; STEP IN (SLOW RATE)
CREADS	EQU	80H	; READ SECTOR
CTERM	EQU	0D0H	; TERMINATE COMMAND (FORCE INT.)
;
; DISK CONTROLLER STATUS - TYPE 1 COMMANDS
;
MNRDY	EQU	80H	; NOT READY
MWPROT	EQU	40H	; WRITE PROTECTED
MHLT	EQU	20H	; HEAD LOADED
MSKERR	EQU	10H	; SEEK ERROR
MCRC	EQU	8	; CRC ERROR
MTZERO	EQU	4	; TRACK ZERO
MINDEX	EQU	2	; INDEX
MBUSY	EQU	1	; CONTROLLER BUSY
;
; DISK CONTROLLER STATUS - TYPE 2 AND 3 COMMANDS
;
MWRFLT EQU	20H	; WRITE FAULT
MRNF	EQU	10H	; RECORD NOT FOUND
MLDATA	EQU	4	; LOST DATA
MDR	EQU	2	; DATA REQUEST
;
; ERROR MASKS FOR OPERATIONS
;
MREAD	EQU	MNRDY+MRNF+MCRC+MLDATA ; READ SECTOR OPN.
MSEEK	EQU	MNRDY+MSKERR+MCRC  ; POSITIONING OPN.
;
; MISCELLANEOUS EQUATES
;
MONIT	EQU	38h	; monitor entry point
MDISROM EQU	1	; ROM DISABLE BIT
MON	EQU	8	; MOTOR ON BIT (PSTAT)
MLEDOFF EQU	0F0H	; BITS TO TURN LEDS OFF
LSTACK	EQU	4100H	; STACK LOCATION
;
;
;==============================================================
;
;
	ORG	4000H
;
BT00:
	DI			; DISABLE INTERRUPTS
	LXI	SP,LSTACK	; SET UP STACK POINTER
BT10:
	IN	PCOMD		; GET FDC STATUS
	ANI	MNRDY+MBUSY	; READY FOR COMMAND
	JNZ	BT10		;   NOT YET - WAIT
;
; RESTORE DISK AND INITIALIZE FOR READING
;
	MVI	A,CREST
	CALL	BTCMD		; RESTORE DISK
	MVI	C,PDATA	        ; SET UP DATA PORT NR
	MVI	E,KNSECTS	; SET UP NUMBER OF SECTORS TO LOAD
	LXI	H,LCCP-128	; SET INITIAL LOAD ADDRESS
	MVI	A,1		; STARTING SECTOR FOR TRACK ZERO
;
; MAIN LOOP - READS AND LOADS SECTORS
;
BT20:
	STA	BTSEC		; STORE CURRENT SECTOR NR.
	CALL	BTREAD		; READ A SECTOR
	IN	PCOMD		; GET FDC STATUS
	ANI	MREAD		; ANY ERRORS
	JNZ	BT90		;   YES
	DCR	E		; LOADED ALL SECTORS
	JZ	BT50		;  YES
	LDA	BTSEC		; GET CURRENT SECTOR NR.
	INR	A		; INCREMENT IT
	CPI	KSECTRK+1	; READ LAST SECTOR OF TRACK
	JC	BT20		;   NOT YET - READ NEXT
	MVI	A,CSTEPIN
	CALL	BTCMD		; STEP IN TO NEXT TRACK
	IN	PCOMD		; GET FDC STATUS
	ANI	MSEEK		; ANY ERRORS
	JNZ	BT90		;   YES
	MVI	A,1		; SET SECTOR 1
	JMP	BT20		; GO LOAD NEXT SECTOR
;
; TURN OFF MOTOR AND JUMP TO BIOS
;
BT50:
	IN	PSTAT		; GET CONTROL BYTE
	MVI	A,MDISROM+MLEDOFF
	OUT	PCONT		; DISABLE ROM AND TURN LEDS OFF
	IN	PSTAT
	ANI	03
	STA	CDISK		; save disk booted from
	JMP	LBIOS		; BRANCH TO BIOS
;
; HANDLE ERROR CONDITIONS - RESTART BT
;
BT90:
	MVI	A,CTERM
	OUT	PCOMD		; TERMINATE COMMAND
	JMP	MONIT		; return to monitor
;
; SUBROUTINE BTCMD
;    SEND COMMAND (NO DATA) TO FDC AND WAIT FOR COMPLETION
;    COMMAND IS IN REG. A
;
BTCMD:
	OUT	PCOMD		; SEND COMMAND TO FDC
BTCMD1:
	IN	PSTAT		; GET GENERAL STATUS
	ADD	A		; SHIFT INT. BIT TO SIGN
	RM			; RETURN ON INTERRUPT
	JMP	BTCMD1		; WAIT FOR COMPLETION
;
;
; SUBROUTINE BOOTREAD
;    READ A SECTOR INTO (HL)
;    SECTOR NUMBER IS IN REG. A
;
BTREAD:
	OUT	PSECRG		; WRITE SECTOR NUMBER
	MVI	A,CREADS
	OUT	PCOMD		; SEND READ SECTOR COMMAND
BTRD10:
	IN	PSTAT		; GET GENERAL STATUS
	ADD	A		; SHIFT INT. BIT TO SIGN
	RM			; RETURN ON INTERRUPT
	JNC	BTRD10		; LOOP IF NO DR
	INI			; LOAD A BYTE
	JMP	BTRD10		; GO WAIT FOR NEXT BYTE
;
;
; TEMPORARY STORAGE
	;
BTSEC   DS	1		; CURRENT SECTOR NUMBER
	;
	END


